// shape.cc

#include "shape.h"
#include <iostream>
#include <fstream>

namespace CommonTilton
{

 // Basic constructor
  Shape::Shape()
  {
    shape_flag = false;
    nb_entities = false;

    X = NULL;
    Y = NULL;

    return;
  }

 // Destructor...
  Shape::~Shape() 
  {
  }

#ifdef SHAPEFIL
 // Open shape data
  void Shape::open(const string& file_name)
  {
    shape_flag = false;

    hSHP = SHPOpen(file_name.c_str(),"rb");
    SHPGetInfo(hSHP, &nb_entities, &shapeType, &minBound[0], &maxBound[0]);
    if (shapeType != SHPT_POINT)
    {
      cout << "ERROR(shape): Shape type must be SHPT_POINT" << endl;
      return;
    }
    X = new double[nb_entities];
    Y = new double[nb_entities];

    hDBF = DBFOpen(file_name.c_str(),"rb");
    nb_records = DBFGetRecordCount(hDBF);
    if (nb_records != nb_entities)
    {
      cout << "ERROR(shape): Number of entities/records mismatch between Shape file and DBF file" << endl;
      return;
    }

    char *field_name;
    field_name = new char[10];
    field_name = "Site_Type";
    site_type_field_index = DBFGetFieldIndex(hDBF,field_name);
    if (site_type_field_index == -1)
    {
      cout << "ERROR(shape): Site Type Field not found in DBF file" << endl;
      return;
    }

    shape_flag = true;

    return;
  }

 // Read shape data
  void Shape::read(const string& site_type)
  {
    SHPObject *shpObject;
    int record;

    nb_entities = 0;
    for (record = 0; record < nb_records; record++)
    {
      if (strcmp(DBFReadStringAttribute(hDBF,record,site_type_field_index),site_type.c_str()) == 0)
      {
        shpObject = SHPReadObject(hSHP, record);
        X[nb_entities] = shpObject->padfX[0];
        Y[nb_entities] = shpObject->padfY[0];
        nb_entities++;
      }
    }
    data_flag = true;

    return;
  }

 // Close shape data file
  void Shape::close()
  {
    SHPClose(hSHP);

    return;
  }
#else
 // Read shape data from an ASCII file
  void Shape::ascii_read(const string& file_name)
  {
    int entity;
    ifstream shape_fs;

    shape_fs.open(file_name.c_str());
    if (!shape_fs)
    {
      cout << "Cannot open shape input file " << file_name << endl;
      shape_flag = false;
      return;
    }
    
  // Read shape file data until End-of-File reached
    string tag, tmp_string;
    bool nb_entities_flag = false;
    bool minBound_flag = false;
    bool maxBound_flag = false;
    bool X_flag = false;
    bool Y_flag = false;
    while (!shape_fs.eof())
    {
      shape_fs >> tag;
      if (tag == "-nb_entities")
      {
        shape_fs >> nb_entities;
        nb_entities_flag = true;
        X = new double[nb_entities];
        Y = new double[nb_entities];
      }
      if (tag == "-minBound")
      {
        for (entity = 0; entity < 4; entity++)
          shape_fs >> minBound[entity];
        minBound_flag = true;
      }
      if (tag == "-maxBound")
      {
        for (entity = 0; entity < 4; entity++)
          shape_fs >> maxBound[entity];
        maxBound_flag = true;
      }
      if (tag == "-X")
      {
        if (nb_entities_flag)
        {
          for (entity = 0; entity < nb_entities; entity++)
            shape_fs >> X[entity];
          X_flag = true;
        }
        else
        {
          cout << "Must specify the number of entities (nb_entities) before specifying \"X\"" << endl;
          shape_flag = false;
          return;
        }
      }
      if (tag == "-Y")
      {
        if (nb_entities_flag)
        {
          for (entity = 0; entity < nb_entities; entity++)
            shape_fs >> Y[entity];
          Y_flag = true;
        }
        else
        {
          cout << "Must specify the number of entities (nb_entities) before specifying \"X\"" << endl;
          shape_flag = false;
          return;
        }
      }
    }
    shape_fs.close();

    data_flag = shape_flag = ((nb_entities_flag) && (minBound_flag) && (maxBound_flag) && (X_flag) && (Y_flag));
  
    return;
  }
#endif

 // Print the shape information
  void Shape::print()
  {
    int entity;

    if (shape_flag)
      cout << "Shape data is valid." << endl;
    else
      cout << "Shape data is invalid." << endl;

    cout.precision(12);
    cout << "nb_entities = " << nb_entities << endl;
    cout << "minBound: ";
    for (entity = 0; entity < 4; entity++)
      cout << minBound[entity] << " ";
    cout << endl;
    cout << "maxBound: ";
    for (entity = 0; entity < 4; entity++)
      cout << maxBound[entity] << " ";
    cout << endl;
    cout << "X: ";
    for (entity = 0; entity < nb_entities; entity++)
      cout << X[entity] << " ";
    cout << endl;
    cout << "Y: ";
    for (entity = 0; entity < nb_entities; entity++)
      cout << Y[entity] << " ";
    cout << endl;

    return;
  }
 // Write shape data in ASCII format
  void Shape::ascii_write(const string& file_name)
  {
    int entity;
    ofstream shape_fs;

    shape_fs.open(file_name.c_str());
    shape_fs.precision(12);
    shape_fs << "-nb_entities " << nb_entities << endl;
    shape_fs << "-minBound ";
    for (entity = 0; entity < 4; entity++)
      shape_fs << minBound[entity] << " ";
    shape_fs << endl;
    shape_fs << "-maxBound ";
    for (entity = 0; entity < 4; entity++)
      shape_fs << maxBound[entity] << " ";
    shape_fs << endl;
    shape_fs << "-X ";
    for (entity = 0; entity < nb_entities; entity++)
      shape_fs << X[entity] << " ";
    shape_fs << endl;
    shape_fs << "-Y ";
    for (entity = 0; entity < nb_entities; entity++)
      shape_fs << Y[entity] << " ";
    shape_fs << endl;

    shape_fs.close();

    return;
  }

} // namespace CommonTilton


